import {Component, ElementRef, OnDestroy, OnInit, ViewChildren} from "@angular/core";
import {AngularfireService} from "../../services/angularfire.service";
import {User} from "../../model/User";
import {UserService} from "../../services/user.service";
import {Router} from "@angular/router";
import {VideoService} from "../../services/video.service";
import {document} from "ngx-bootstrap/utils/facade/browser";
import {AlertComponent} from "../alert/alert.component";
import {Locale} from "../../constants/Locale";
import {Subject} from "rxjs/Subject";

@Component({
  selector: "app-video-page",
  templateUrl: "./timeline-video.component.html",
  styleUrls: ["./timeline-video.component.css"]
})
export class TimelineVideoComponent implements OnInit, OnDestroy {

  public videoObjectArray: Array<any> = [];

  public time = 0;
  @ViewChildren("videoRef")
  public videoList: any;

  public duration: number;
  public steps: Array<number>;
  public user: User;
  public timeline: any;
  public i = 0;
  public timelineTime: number;
  public instId: string;
  public locale: any = Locale.getLocale();
  private subscriptions: Array<any> = [];
  private timelineTimeSubject: Subject<any> = new Subject<any>();

  constructor(private afService: AngularfireService,
              private userSvc: UserService,
              private videoSvc: VideoService,
              private router: Router) {
    this.timelineTimeSubject = this.videoSvc.getFindItemInTimeline();
  }


  public async ngOnInit() {
    this.user = this.userSvc.user;
    if (this.user === undefined) {
      this.router.navigate(["login"]);
      return;
    }
    this.videoObjectArray = [];
    // registrace videa
    const videoSub = this.videoSvc.getVideoSub().subscribe((object: any) => {
      const test = object.item;
      const instId = object.instId;
      if (instId !== this.instId) {
        return;
      }
      // zalozim objekt, do ktereho pozdeji budu pridavat data
      const obj = {id: test.id, name: test.name, timestamp: 0, url: "", curr: 0, duration: 0, preview: ""};
      const contains = this.videoObjectArray.find(x => x.id === test.id);
      if (contains) {
        AlertComponent.error(Locale.getLocale().alertVideoIsIn);
        return;
      }
      this.videoObjectArray.push(obj);
      const videoData = this.afService.getAbstractData(test.id);
      const subscription = videoData.subscribe((data): any => {
        if (!data) {
          return;
        }
        let ts = Number.parseInt(data["timestamp"]);
        ts = isNaN(ts) ? 0 : ts;
        obj.timestamp = ts;
        // pokud neni hotove videa, tak muzou byt k dispozici preview fotky, zobrazim je

        if (data && data["preview"] && data["preview"]["downloadUrl"]) {
          obj.preview = data["preview"]["downloadUrl"];
        }

        // pokud je nahrane cele video, muzu ho stahnout a nacist
        if (data["video"] && data["video"]["status"].toUpperCase() === "FINISHED") {
          obj.url = data["video"]["downloadUrl"];
          const vid: any = document.getElementById(obj.id);
          obj.preview = "";
          if (vid) {
            vid.load();
          }
        }
      });
    });

    // registrace currentTime, kdyz se odjinund (timeline) zavola funkce next() s daty,
    // tadle funkce se spusti
    const timeSub = this.videoSvc.getTimelineSub().subscribe((data: any) => {
      // ovladam jen videa, ktera patri ke sve instanci
      if (data.instId !== this.instId) {
        return;
      }
      if (!this.videoList) {
        return;
      }
      this.timelineTime = data.time;

      // prochazim videa z DOMu
      for (const elRef of this.videoList._results) {
        const el = elRef.nativeElement;
        const video = this.videoObjectArray.find(x => x.id === el.id);
        if (!el) {
          continue;
        }
        el.playbackRate = data["speed"] ? data["speed"] : el.playbackRate;
        const ts = video.timestamp;
        const videoCurrentTime = ts + el.currentTime * 1000;
        const videoMaxTime = ts + el.duration * 1000;
        const diff = this.timelineTime - videoCurrentTime;
        // pokud video ma prave hrat
        if (ts <= this.timelineTime && this.timelineTime <= videoMaxTime) {
          if (data.play) {
            el.play();
            const offset = 1000;
            // pokud je aktualni hodnota videa o hodne mimo (1 sec), nez ma byt, tak se srovna
            if (videoCurrentTime + offset < this.timelineTime ||
              videoCurrentTime - offset > this.timelineTime) {
              console.log("synchronizace casu");
              // el.pause();
              el.currentTime = (this.timelineTime - ts) / 1000;
            }
            // zpetna vazba. v dalsim tiku si timeline vezme tuto hodnotu a zmensi delku tiku
            this.videoSvc.videoOffset = diff;
          } else {
            el.pause();
            el.currentTime = (this.timelineTime - ts) / 1000;
          }
        } else {
          // pokud je cas casove osy mimo dobu, kdy ma video hrat, tak se zastavi
          el.pause();
          el.currentTime = (this.timelineTime - ts) / 1000;
        }
        video.curr = el.currentTime * 1000;
        video.duration = el.duration * 1000;
      }
    });

    this.subscriptions.push(videoSub);
    this.subscriptions.push(timeSub);
  }

  /**
   * nastavi cas timeline na cas fotky
   */
  public findInTimeline(video: any): void {
    this.timelineTimeSubject.next(Number.parseInt(video["timestamp"]));
  }

  public mute(video: any): void {
    const el = document.getElementById(video.id);
    if (el) {
      el.muted = !el.muted;
    }
  }

  /**
   * zavre video
   * @param video
   */
  public close(video): void {
    console.log(video);
    this.videoObjectArray = this.videoObjectArray.filter(item => item !== video);
  }

  /**
   * zvetseni/zmenseni videa/preview fotky
   * @param pixels
   * @param video
   * @param isVideo
   */
  public zoom(pixels: number, video: ElementRef, isVideo: boolean = true): void {
    console.log("change size");
    let w: any = video["style"]["width"];
    w = w.substring(0, w.length - 2);
    w = Number.parseInt(w);
    w += pixels;
    if (isVideo) {
      video["style"]["height"] = w + "px";
    }
    video["style"]["width"] = w + "px";
  }


  ngOnDestroy(): void {
    console.log("destroying video");
    this.subscriptions.forEach(sub => {
      sub.unsubscribe();
    });
  }
}
